;***********************************************************************
;solve the Knight's tour using Warnsdorff's rule (always
;moving to the square having the fewest onward moves)
;V1.00 (06/15/1975): original for Univac 1108
;V1.10 (09/21/1981): revised for IBM PC
;***********************************************************************
STACK   SEGMENT PARA STACK 'STACK'
        DW      32 DUP(0)
STACK   ENDS
;***********************************************************************
DATA    SEGMENT PARA PUBLIC 'DATA'
B       DW      0
BOARD   DB      1,63 DUP(0)
BOARD1  DB      '',0
BOARD2  DB      '',0
BOARD3  DB      '??',0
BOARD4  DB      '??',0
CRLF    DB      13,10,0
ERROR   DB      'SOLUTION NOT FOUND',7,13,10,0
FOUND   DB      1
HELP1   DB      'KNIGHT''S TOUR',0
HELP2   DB      'use the  keys to select the',0
HELP3   DB      'knight''s first position',0
HELP4   DB      'press F1 to solve',0
HELP5   DB      'press Esc to exit',0
M       DB      0
MOVE    DW      102H,201H,2FFH,1FEH,0FFFEH,0FEFFH,0FE01H,0FF02H,0
N       DB      0
XY1     DW      0
XY3     DW      0
DATA    ENDS
;***********************************************************************
;load address and write string
;***********************************************************************
WRITES  MACRO   STRING
        MOV     SI,OFFSET STRING
        CALL    WRITE
        ENDM
;***********************************************************************
;move cursor, load address, and write string
;***********************************************************************
WRITEX  MACRO   ROW,COL,STRING
        MOV     AH,0FH
        INT     10H
        MOV     DH,ROW
        MOV     DL,COL
        MOV     AH,2
        INT     10H
        WRITES  STRING
        ENDM
;***********************************************************************
CODE    SEGMENT PARA PUBLIC 'CODE'
        ASSUME CS:CODE,DS:DATA
;
MAIN:   MOV     AX,DATA
        MOV     DS,AX
        MOV     ES,AX
;
        MOV     AH,5
        MOV     AL,1
        INT     10H
;
M100:   CALL    SHOW
        CALL    ASKME
        CMP     AL,27
        JE      M110
        CALL    SETUP
        CALL    SOLVE
        JMP     M100
;
M110:   MOV     AH,5
        MOV     AL,0
        INT     10H
        CALL    EXIT
;***********************************************************************
;terminate program
;***********************************************************************
EXIT    PROC    NEAR
        MOV     AH,51H
        INT     21H
        PUSH    BX
        MOV     BX,0
        PUSH    BX
        RETF
EXIT    ENDP
;***********************************************************************
;write a string on the screen
;***********************************************************************
WRITE   PROC    NEAR
        MOV     AH,0FH
        INT     10H
W100:   MOV     AL,DS:[SI]
        CMP     AL,0
        JE      W101
        MOV     AH,0EH
        INT     10H
        INC     SI
        JMP     W100
W101:   RET
WRITE   ENDP
;***********************************************************************
;initialize board
;***********************************************************************
SETUP   PROC    NEAR
        MOV     AL,0
        MOV     CX,64
        MOV     DI,OFFSET BOARD
        PUSH    DI
        REP     STOSB
        POP     DI
        ADD     DI,B
        MOV     DS:BYTE PTR[DI],1
        MOV     AX,B
        MOV     DL,8
        DIV     DL
        MOV     XY1,AX
        MOV     M,0
        RET
SETUP   ENDP
;***********************************************************************
;get pointer to BOARD[CH][CL]
;***********************************************************************
GETPTR  PROC    NEAR
        MOV     BL,CL
        SHL     BL,1
        SHL     BL,1
        SHL     BL,1
        ADD     BL,CH
        MOV     BH,0
        ADD     BX,OFFSET BOARD
        RET
GETPTR  ENDP
;***********************************************************************
;check for move OK
;***********************************************************************
MOVEOK  PROC    NEAR
        CMP     CH,0
        JL      M200
        CMP     CH,7
        JG      M200
        CMP     CL,0
        JL      M200
        CMP     CL,7
        JG      M200
        CALL    GETPTR
        CMP     DS:BYTE PTR[BX],0
        JNE     M200
        CLC
        RET
M200:   STC
        RET
MOVEOK  ENDP
;***********************************************************************
;count number of moves
;***********************************************************************
MOVES   PROC    NEAR
        MOV     AL,0
        MOV     SI,OFFSET MOVE
M300:   MOV     CX,DS:[SI]
        ADD     SI,2
        CMP     CX,0
        JE      M301
        ADD     CH,DH
        ADD     CL,DL
        CALL    MOVEOK
        JC      M300
        INC     AL
        JMP     M300
M301:   RET
MOVES   ENDP
;***********************************************************************
;solve
;***********************************************************************
SOLVE   PROC    NEAR
;
S100:	MOV	N,9
        MOV     DI,OFFSET MOVE
S101:   MOV     DX,DS:[DI]
        ADD     DI,2
        CMP     DX,0
        JE      S110
;
        MOV     CX,XY1
        ADD     CH,DH
        ADD     CL,DL
        MOV     DX,CX
        CALL    MOVEOK
        JC      S101
;
        CMP     M,62
        JNE     S102
        MOV     XY3,DX
        JMP     S110
;
S102:   CALL    GETPTR
        MOV     AL,M
        ADD     AL,2
        MOV     DS:[BX],AL
        PUSH    BX
        CALL    MOVES
        POP     BX
        MOV     DS:BYTE PTR[BX],0
;
        CMP     AL,1
        JL      S101
        CMP     AL,N
        JGE     S101
        MOV     N,AL
        MOV     XY3,DX
        JMP     S101
;
S110:   CMP     N,9
        JNE     S111
        CMP     M,62
        JE      S111
        MOV     FOUND,0
        JMP     S120
;
S111:   INC     M
        MOV     CX,XY3
        MOV     XY1,CX
        CALL    GETPTR
        MOV     AL,M
        INC     AL
        MOV     DS:[BX],AL
;
        CMP     M,63
        JE      S120
        MOV     FOUND,1
        JMP     S100
;
S120:   RET
SOLVE   ENDP
;***********************************************************************
;show board on screen
;***********************************************************************
SHOW    PROC    NEAR
;
        PUSH    ES
        MOV     AX,0B900H
        MOV     ES,AX
        MOV     DI,0
        MOV     AX,720H
        MOV     CX,80*25
        REP     STOSW
        POP     ES
;
        MOV     AH,2
        MOV     BH,1
        MOV     DX,0
        INT     10H
;
        MOV     CL,7
;
S200:   CALL    S210
        MOV     CH,0
S201:   CALL    GETPTR
        CALL    S230
        CALL    GETPTR
        CALL    S240
        CMP     CH,8
        JL      S201
        WRITES  CRLF
        CALL    S210
;
        DEC     CL
        CALL    S220
        MOV     CH,0
S202:   CALL    GETPTR
        CALL    S240
        CALL    GETPTR
        CALL    S230
        CMP     CH,8
        JL      S202
        WRITES  CRLF
        CALL    S220
;
        SUB     CL,1
        JNC     S200
        RET
;
S210:   MOV     CH,0
S211:   WRITES  BOARD1
        INC     CH
        WRITES  BOARD2
        INC     CH
        CMP     CH,8
        JL      S211
        WRITES  CRLF
        RET
;
S220:   MOV     CH,0
S221:   WRITES  BOARD2
        INC     CH
        WRITES  BOARD1
        INC     CH
        CMP     CH,8
        JL      S221
        WRITES  CRLF
        RET
;
S230:   CALL    S250
        MOV     BOARD3[2],AL
        MOV     BOARD3[3],AH
        WRITES  BOARD3
        INC     CH
        RET
;
S240:   CALL    S250
        MOV     BOARD4[2],AL
        MOV     BOARD4[3],AH
        WRITES  BOARD4
        INC     CH
        RET
;
S250:   MOV     AL,DS:[BX]
        MOV     AH,0
        MOV     DL,10
        DIV     DL
        ADD     AL,'0'
        ADD     AH,'0'
        RET
;
SHOW    ENDP
;***********************************************************************
;user input
;***********************************************************************
ASKME   PROC    NEAR
;
        WRITEX  0,50,HELP1
        WRITEX  2,50,HELP2
        WRITEX  3,50,HELP3
        WRITEX  5,50,HELP4
        WRITEX  6,50,HELP5
        CMP     FOUND,0
        JNE     A100
        WRITEX  23,50,ERROR
;
A100:   MOV     AX,B
        MOV     DL,8
        DIV     DL
        XCHG    AL,AH
        MOV     DL,AL
        MOV     AL,7
        SUB     AL,AH
        MOV     AH,3
        MUL     AH
        ADD     AL,1
        MOV     DH,AL
        MOV     AL,DL
        MOV     AH,6
        MUL     AH
        ADD     AL,3
        MOV     DL,AL
        MOV     BH,1
        MOV     AH,2
        INT     10H
;
        PUSH    ES
        MOV     AX,40H
        MOV     ES,AX
        AND     ES:BYTE PTR[17H],0DFH
        POP     ES
        MOV     AH,0
        INT     16H
;
        CMP     AX,4800H
        JNE     A120
        ADD     B,8
        CMP     B,63
        JLE     A110
        SUB     B,64
A110:   JMP     A100
;
A120:   CMP     AX,5000H
        JNE     A130
        SUB     B,8
        JNC     A121
        ADD     B,64
A121:   JMP     A100
;
A130:   CMP     AX,4D00H
        JNE     A140
        ADD     B,1
        TEST    B,7
        JNZ     A131
        SUB     B,8
A131:   JMP     A100
;
A140:   CMP     AX,4B00H
        JNE     A150
        SUB     B,1
        JNC     A141
        ADD     B,8
        JMP     A142
A141:   MOV     AX,B
        AND     AX,7
        CMP     AX,7
        JNE     A131
        ADD     B,8
A142:   JMP     A100
;
A150:   CMP     AX,3B00H
        JE      A160
        CMP     AL,27
        JE      A160
        MOV     AH,0EH
        MOV     AL,7
        INT     10H
        JMP     A100
;
A160:   RET
ASKME   ENDP
;***********************************************************************
CODE    ENDS
END     MAIN
